<?php

/**
 * @file
 * Contains \Drupal\textimage\Plugin\ImageEffect\TextimageText.
 */

namespace Drupal\textimage\Plugin\ImageEffect;

use Drupal\Component\Utility\NestedArray;
use Drupal\Component\Utility\Unicode;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Image\ImageInterface;
use Drupal\Core\Render\BubbleableMetadata;
use Drupal\textimage\Component\ColorUtility;
use Drupal\textimage\Component\Rectangle;
use Drupal\textimage\Element\TextimageColor;

/**
 * Define the Textimage text.
 *
 * @ImageEffect(
 *   id = "textimage_text",
 *   label = @Translation("Textimage text"),
 *   description = @Translation("Define text font, size and positioning.")
 * )
 */
class TextimageText extends TextimageEffectBase {

  // $info stores information about image and text wrapper.
  protected $info = [
    'image_xpos' => 0,
    'image_ypos' => 0,
  ];

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return NestedArray::mergeDeep(
      array(
        'font'          => array(
          'name'                  => \Drupal::service('config.factory')->get('textimage.settings')->get('default_font.name'),
          'uri'                   => \Drupal::service('config.factory')->get('textimage.settings')->get('default_font.uri'),
          'size'                  => 16,
          'angle'                 => 0,
          'color'                 => '#000000FF',
          'stroke_mode'           => 'outline',
          'stroke_color'          => '#000000FF',
          'outline_top'           => 0,
          'outline_right'         => 0,
          'outline_bottom'        => 0,
          'outline_left'          => 0,
          'shadow_x_offset'       => 1,
          'shadow_y_offset'       => 1,
          'shadow_width'          => 0,
          'shadow_height'         => 0,
        ),
        'layout'       => array(
          'padding_top'           => 0,
          'padding_right'         => 0,
          'padding_bottom'        => 0,
          'padding_left'          => 0,
          'x_pos'                 => 'center',
          'y_pos'                 => 'center',
          'x_offset'              => 0,
          'y_offset'              => 0,
          'background_color'      => NULL,
          'overflow_action'       => 'extend',
        ),
        'text' => array(
          'maximum_width'         => 0,
          'fixed_width'           => FALSE,
          'align'                 => 'left',
          'line_spacing'          => 0,
          'case_format'           => '',
        ),
        'text_string'             => $this->t('Preview'),
      ),
      parent::defaultConfiguration()
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = array();

    // --- Preview effect.
    $this->configuration['preview_bar']['debug_visuals'] = empty($this->configuration['preview_bar']['debug_visuals']) ? FALSE : TRUE;
    $form['preview'] = array(
      '#type'   => 'item',
      '#title' => $this->t('Preview'),
      '#theme' => 'textimage_preview',
      '#textimage_data' => $this->configuration,
    );

    // --- Preview bar.
    $form['preview_bar'] = array(
      '#type' => 'container',
      '#attributes' => array(
        'class' => array('container-inline'),
      ),
    );
    // Refresh button.
    $form['preview_bar']['preview'] = array(
      '#type'  => 'button',
      '#value' => $this->t('Refresh preview'),
      '#name' => 'preview',
      '#ajax'  => array(
        'callback' => array($this, 'processAjaxPreview'),
      ),
    );
    // Visual aids.
    $form['preview_bar']['debug_visuals'] = array(
      '#type' => 'checkbox',
      '#title' => $this->t('Visual aids in preview'),
      '#default_value' => FALSE,
    );

    // --- Settings.
    $form['settings'] = array(
      '#type' => 'vertical_tabs',
      '#tree' => FALSE,
    );

    // --- Text default.
    $form['text_default'] = array(
      '#type'  => 'details',
      '#title' => $this->t('Text default'),
      '#group'   => 'settings',
    );
    $form['text_default']['text_string'] = array(
      '#type'  => 'textarea',
      '#title' => $this->t('Default text'),
      '#default_value' => $this->configuration['text_string'],
      '#description' => $this->t('Enter the default text string for this effect. You can also enter tokens, that Textimage will resolve when applying the effect.'),
      '#rows' => 3,
      '#required' => TRUE,
    );
    if ($this->moduleHandler->moduleExists('token')) {
      $form['text_default']['tokens'] = array(
        '#theme' => 'token_tree',
        '#token_types' => array('node', 'user', 'file', 'textimage'),
        '#global_types' => TRUE,
        '#click_insert' => TRUE,
      );
    }

    // ---- Font settings.
    $form['font'] = array(
      '#type'  => 'details',
      '#title' => $this->t('Font settings'),
      '#group'   => 'settings',
    );
    $form['font']['name'] = $this->fontPlugin->selectionElement(array(
      '#title' => $this->t('Font'),
      '#description' => $this->t('Select the font to be used in this image.'),
      '#default_value' => $this->configuration['font']['name'],
    ));
    $form['font']['size'] = array(
      '#type'  => 'number',
      '#title' => $this->t('Size'),
      '#description'   => $this->t('Enter the size of the text to be generated.'),
      '#default_value' => $this->configuration['font']['size'],
      '#maxlength' => 5,
      '#size' => 3,
      '#required' => TRUE,
      '#min' => 1,
    );
    $form['font']['angle'] = array(
      '#type'  => 'number',
      '#title' => $this->t('Rotation'),
      '#maxlength' => 4,
      '#size' => 4,
      '#field_suffix' => $this->t('&deg;'),
      '#description' => $this->t('Enter the angle in degrees at which the text will be displayed. Positive numbers rotate the text clockwise, negative numbers counter-clockwise.'),
      '#default_value' => $this->configuration['font']['angle'],
      '#min' => -360,
      '#max' => 360,
    );
    $form['font']['color'] = array(
      '#type' => 'textimage_color',
      '#title' => $this->t('Font color'),
      '#description'  => $this->t('Set the font color.'),
      '#allow_opacity' => TRUE,
      '#default_value' => $this->configuration['font']['color'],
    );
    // Outline.
    $form['font']['stroke'] = array(
      '#type' => 'details',
      '#title' => $this->t('Outline / Shadow'),
      '#description'   => $this->t('Optionally add an outline or shadow around the font. Enter the information in pixels.'),
    );
    $stroke_options = array(
      'outline' => $this->t('Outline'),
      'shadow' => $this->t('Shadow'),
    );
    $form['font']['stroke']['mode'] = array(
      '#type'    => 'radios',
      '#title'   => $this->t('Mode'),
      '#options' => $stroke_options,
      '#default_value' => $this->configuration['font']['stroke_mode'],
    );
    $form['font']['stroke']['top'] = array(
      '#type' => 'number',
      '#title' => $this->t('Top'),
      '#default_value' => $this->configuration['font']['outline_top'],
      '#maxlength' => 2,
      '#size' => 3,
      '#field_suffix' => 'px',
      '#min' => 0,
      '#states' => array(
        'visible' => array(
          ':radio[name="data[font][stroke][mode]"]' => array('value' => 'outline'),
        ),
      ),
    );
    $form['font']['stroke']['right'] = array(
      '#type' => 'number',
      '#title' => $this->t('Right'),
      '#default_value' => $this->configuration['font']['outline_right'],
      '#maxlength' => 2,
      '#size' => 3,
      '#field_suffix' => 'px',
      '#min' => 0,
      '#states' => array(
        'visible' => array(
          ':radio[name="data[font][stroke][mode]"]' => array('value' => 'outline'),
        ),
      ),
    );
    $form['font']['stroke']['bottom'] = array(
      '#type' => 'number',
      '#title' => $this->t('Bottom'),
      '#default_value' => $this->configuration['font']['outline_bottom'],
      '#maxlength' => 2,
      '#size' => 3,
      '#field_suffix' => 'px',
      '#min' => 0,
      '#states' => array(
        'visible' => array(
          ':radio[name="data[font][stroke][mode]"]' => array('value' => 'outline'),
        ),
      ),
    );
    $form['font']['stroke']['left'] = array(
      '#type' => 'number',
      '#title' => $this->t('Left'),
      '#default_value' => $this->configuration['font']['outline_left'],
      '#maxlength' => 2,
      '#size' => 3,
      '#field_suffix' => 'px',
      '#min' => 0,
      '#states' => array(
        'visible' => array(
          ':radio[name="data[font][stroke][mode]"]' => array('value' => 'outline'),
        ),
      ),
    );
    $form['font']['stroke']['x_offset'] = array(
      '#type' => 'number',
      '#title' => $this->t('Horizontal offset'),
      '#default_value' => $this->configuration['font']['shadow_x_offset'],
      '#maxlength' => 3,
      '#size' => 3,
      '#field_suffix' => 'px',
      '#min' => 1,
      '#states' => array(
        'visible' => array(
          ':radio[name="data[font][stroke][mode]"]' => array('value' => 'shadow'),
        ),
      ),
    );
    $form['font']['stroke']['y_offset'] = array(
      '#type' => 'number',
      '#title' => $this->t('Vertical offset'),
      '#default_value' => $this->configuration['font']['shadow_y_offset'],
      '#maxlength' => 3,
      '#size' => 3,
      '#field_suffix' => 'px',
      '#min' => 1,
      '#states' => array(
        'visible' => array(
          ':radio[name="data[font][stroke][mode]"]' => array('value' => 'shadow'),
        ),
      ),
    );
    $form['font']['stroke']['width'] = array(
      '#type' => 'number',
      '#title' => $this->t('Horizontal elongation'),
      '#default_value' => $this->configuration['font']['shadow_width'],
      '#maxlength' => 2,
      '#size' => 3,
      '#field_suffix' => 'px',
      '#min' => 0,
      '#states' => array(
        'visible' => array(
          ':radio[name="data[font][stroke][mode]"]' => array('value' => 'shadow'),
        ),
      ),
    );
    $form['font']['stroke']['height'] = array(
      '#type' => 'number',
      '#title' => $this->t('Vertical elongation'),
      '#default_value' => $this->configuration['font']['shadow_height'],
      '#maxlength' => 2,
      '#size' => 3,
      '#field_suffix' => 'px',
      '#min' => 0,
      '#states' => array(
        'visible' => array(
          ':radio[name="data[font][stroke][mode]"]' => array('value' => 'shadow'),
        ),
      ),
    );
    $form['font']['stroke']['color'] = array(
      '#type' => 'textimage_color',
      '#title' => $this->t('Color'),
      '#description'  => $this->t('Set the outline/shadow color.'),
      '#allow_opacity' => TRUE,
      '#default_value' => $this->configuration['font']['stroke_color'],
    );

    // ---- Text settings.
    $form['text'] = array(
      '#type'  => 'details',
      '#title' => $this->t('Text settings'),
      '#group'   => 'settings',
    );
    // Inner width.
    $form['text']['maximum_width'] = array(
      '#type'  => 'number',
      '#title' => $this->t('Maximum width'),
      '#field_suffix' => $this->t('px'),
      '#description' => $this->t('Maximum width of the text image, inclusive of padding. Text lines wider than this will be wrapped. Set to 0 to disable wrapping. <b>Note:</b> in case of rotation, the width of the final image rendered will differ, to accomodate the rotation. If you need a strict width/height, add image resize/scale/crop effects afterwards.'),
      '#default_value' => $this->configuration['text']['maximum_width'],
      '#maxlength' => 4,
      '#size' => 4,
      '#min' => 0,
    );
    $form['text']['fixed_width'] = array(
      '#type'  => 'checkbox',
      '#title' => $this->t('Fixed width?'),
      '#description' => $this->t('If checked, the width will always be equal to the maximum width.'),
      '#default_value' => $this->configuration['text']['fixed_width'],
      '#states' => array(
        'visible' => array(
          ':input[name="data[text][maximum_width]"]' => array('!value' => 0),
        ),
      ),
    );
    // Text alignment.
    $form['text']['align'] = array(
      '#type'  => 'select',
      '#title' => $this->t('Text alignment'),
      '#options' => array(
        'left' => $this->t('Left'),
        'center' => $this->t('Center'),
        'right' => $this->t('Right'),
      ),
      '#default_value' => $this->configuration['text']['align'],
      '#description' => $this->t('Select how the text should be aligned within the resulting image. The default aligns to the left.'),
    );
    // Line spacing (Leading).
    $form['text']['line_spacing'] = array(
      '#type'  => 'number',
      '#title' => $this->t('Line spacing (Leading)'),
      '#field_suffix'  => $this->t('px'),
      '#default_value' => $this->configuration['text']['line_spacing'],
      '#maxlength' => 4,
      '#size' => 4,
      '#description' => $this->t('Specify the space in pixels to be added between text lines (Leading). Can be negative.'),
    );
    $form['text']['case_format'] = array(
      '#type'  => 'select',
      '#title' => $this->t('Case format'),
      '#options' => array(
        '' => $this->t('Default'),
        'upper' => $this->t('UPPERCASE'),
        'lower' => $this->t('lowercase'),
        'ucwords' => $this->t('Uppercase Words'),
        'ucfirst' => $this->t('Uppercase first'),
      ),
      '#description' => $this->t('Convert the input text to a desired format. The default makes no changes to input text.'),
      '#default_value' => $this->configuration['text']['case_format'],
    );

    // ---- Layout settings.
    $form['layout'] = array(
      '#type'  => 'details',
      '#title' => $this->t('Layout settings'),
      '#group'   => 'settings',
    );
    // Position.
    $form['layout']['position'] = array(
      '#type' => 'details',
      '#open' => TRUE,
      '#title' => $this->t('Position'),
    );
    $form['layout']['position']['placement'] = array(
      '#type' => 'radios',
      '#title' => $this->t('Placement'),
      '#options' => array(
        'left-top' => $this->t('Top left'),
        'center-top' => $this->t('Top center'),
        'right-top' => $this->t('Top right'),
        'left-center' => $this->t('Center left'),
        'center-center' => $this->t('Center'),
        'right-center' => $this->t('Center right'),
        'left-bottom' => $this->t('Bottom left'),
        'center-bottom' => $this->t('Bottom center'),
        'right-bottom' => $this->t('Bottom right'),
      ),
      '#theme' => 'image_anchor',
      '#default_value' => implode('-', array($this->configuration['layout']['x_pos'], $this->configuration['layout']['y_pos'])),
      '#description' => $this->t('Position of the text on the underlying image.'),
    );
    $form['layout']['position']['x_offset'] = array(
      '#type'  => 'number',
      '#title' => $this->t('Horizontal offset'),
      '#field_suffix'  => 'px',
      '#description'   => $this->t('Additional horizontal offset from placement.'),
      '#default_value' => $this->configuration['layout']['x_offset'],
      '#maxlength' => 4,
      '#size' => 4,
    );
    $form['layout']['position']['y_offset'] = array(
      '#type'  => 'number',
      '#title' => $this->t('Vertical offset'),
      '#field_suffix'  => 'px',
      '#description'   => $this->t('Additional vertical offset from placement.'),
      '#default_value' => $this->configuration['layout']['y_offset'],
      '#maxlength' => 4,
      '#size' => 4,
    );
    // Overflow action.
    $form['layout']['position']['overflow_action'] = array(
      '#type' => 'radios',
      '#title' => $this->t('Overflow'),
      '#default_value' => $this->configuration['layout']['overflow_action'],
      '#options' => array(
        'extend' => $this->t('<b>Extend image.</b> The underlying image will be extended to fit the text.'),
        'crop' => $this->t('<b>Crop text.</b> Only the part of the text fitting in the image is rendered.'),
        'scaletext' => $this->t('<b>Scale text.</b> The text will be scaled to fit the underlying image.'),
      ),
      '#description' => $this->t('Action to take if text overflows the underlying image.'),
    );
    // Padding.
    $form['layout']['padding'] = array(
      '#type' => 'details',
      '#open' => TRUE,
      '#title' => $this->t('Padding'),
      '#description' => $this->t('Specify the padding in pixels to be added around the generated text.'),
    );
    $form['layout']['padding']['top'] = array(
      '#type'  => 'number',
      '#title' => $this->t('Top'),
      '#field_suffix'  => $this->t('px'),
      '#default_value' => $this->configuration['layout']['padding_top'],
      '#maxlength' => 4,
      '#size' => 4,
    );
    $form['layout']['padding']['right'] = array(
      '#type'  => 'number',
      '#title' => $this->t('Right'),
      '#field_suffix'  => $this->t('px'),
      '#default_value' => $this->configuration['layout']['padding_right'],
      '#maxlength' => 4,
      '#size' => 4,
    );
    $form['layout']['padding']['bottom'] = array(
      '#type'  => 'number',
      '#title' => $this->t('Bottom'),
      '#field_suffix'  => $this->t('px'),
      '#default_value' => $this->configuration['layout']['padding_bottom'],
      '#maxlength' => 4,
      '#size' => 4,
    );
    $form['layout']['padding']['left'] = array(
      '#type'  => 'number',
      '#title' => $this->t('Left'),
      '#field_suffix'  => $this->t('px'),
      '#default_value' => $this->configuration['layout']['padding_left'],
      '#maxlength' => 4,
      '#size' => 4,
    );
    // Background color.
    $form['layout']['background_color'] = array(
      '#type' => 'textimage_color',
      '#title' => $this->t('Background color'),
      '#description'  => $this->t('Select the color you wish to use for the background of the text.'),
      '#allow_null' => TRUE,
      '#allow_opacity' => TRUE,
      '#default_value' => $this->configuration['layout']['background_color'],
    );

    $form['#attached']['library'][] = 'textimage/admin.ui';

    return $form;
  }

  /**
   * AJAX callback.
   */
  public function processAjaxPreview($form, FormStateInterface $form_state) {
    $preview = array(
      '#theme' => 'textimage_preview',
      '#textimage_data' => $form_state->getValue(['data', 'ajax_config']),
    );
    $response = new AjaxResponse();
    $response->addCommand(new ReplaceCommand('#textimage-preview', $preview));
    return $response;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::validateConfigurationForm($form, $form_state);

    // Get x-y position from the anchor element.
    list($x_pos, $y_pos) = explode('-', $form_state->getValue(['layout', 'position', 'placement']));

    // Get the font URI.
    $font_uri = $form_state->hasValue(['font', 'name']) ? $this->fontPlugin->getUri($form_state->getValue(['font', 'name'])) : NULL;

    $this->configuration = array(
      'font'   => array(
        'name'                 => $form_state->hasValue(['font', 'name']) ? $form_state->getValue(['font', 'name']) : NULL,
        'uri'                  => $font_uri,
        'size'                 => $form_state->getValue(['font', 'size']),
        'angle'                => $form_state->getValue(['font', 'angle']),
        'color'                => $form_state->getValue(['font', 'color']),
        'stroke_mode'          => $form_state->getValue(['font', 'stroke', 'mode']),
        'stroke_color'         => $form_state->getValue(['font', 'stroke', 'color']),
        'outline_top'          => $form_state->getValue(['font', 'stroke', 'top']),
        'outline_right'        => $form_state->getValue(['font', 'stroke', 'right']),
        'outline_bottom'       => $form_state->getValue(['font', 'stroke', 'bottom']),
        'outline_left'         => $form_state->getValue(['font', 'stroke', 'left']),
        'shadow_x_offset'      => $form_state->getValue(['font', 'stroke', 'x_offset']),
        'shadow_y_offset'      => $form_state->getValue(['font', 'stroke', 'y_offset']),
        'shadow_width'         => $form_state->getValue(['font', 'stroke', 'width']),
        'shadow_height'        => $form_state->getValue(['font', 'stroke', 'height']),
      ),
      'layout' => array(
        'padding_top'          => $form_state->getValue(['layout', 'padding', 'top']),
        'padding_right'        => $form_state->getValue(['layout', 'padding', 'right']),
        'padding_bottom'       => $form_state->getValue(['layout', 'padding', 'bottom']),
        'padding_left'         => $form_state->getValue(['layout', 'padding', 'left']),
        'x_pos'                => $x_pos,
        'y_pos'                => $y_pos,
        'x_offset'             => $form_state->getValue(['layout', 'position', 'x_offset']),
        'y_offset'             => $form_state->getValue(['layout', 'position', 'y_offset']),
        'overflow_action'      => $form_state->getValue(['layout', 'position', 'overflow_action']),
        'background_color'     => $form_state->getValue(['layout', 'background_color']),
      ),
      'text'   => array(
        'maximum_width'        => $form_state->getValue(['text', 'maximum_width']),
        'fixed_width'          => $form_state->getValue(['text', 'fixed_width']),
        'align'                => $form_state->getValue(['text', 'align']),
        'case_format'          => $form_state->getValue(['text', 'case_format']),
        'line_spacing'         => $form_state->getValue(['text', 'line_spacing']),
      ),
      'text_string'            => $form_state->getValue(['text_default', 'text_string']),
    );

// @todo only on ajax
$form_state->setValue(['ajax_config'], $this->configuration);
$form_state->setValue(['ajax_config', 'preview_bar', 'debug_visuals'], $form_state->getValue(['preview_bar', 'debug_visuals']));
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
  }

  /**
   * {@inheritdoc}
   */
  public function getSummary() {
    $data = $this->configuration;
    $data['font_color_detail'] = array(
      '#theme' => 'textimage_color_detail',
      '#color' => $data['font']['color'],
      '#border' => TRUE,
      '#border_color' => 'matchLuma',
    );
    if ($stroke_mode = $this->strokeMode()) {
      $data['stroke_mode'] = $stroke_mode;
      $data['stroke_color_detail'] = array(
        '#theme' => 'textimage_color_detail',
        '#color' => $data['font']['stroke_color'],
        '#border' => TRUE,
        '#border_color' => 'matchLuma',
      );
    }
    if ($data['layout']['background_color']) {
      $data['background_color_detail'] = array(
        '#theme' => 'textimage_color_detail',
        '#color' => $data['layout']['background_color'],
        '#border' => TRUE,
        '#border_color' => 'matchLuma',
      );
    }

    return array(
      '#theme' => 'textimage_text_summary',
      '#data' => $data,
    ) + parent::getSummary();
  }

  /**
   * {@inheritdoc}
   */
  public function applyEffect(ImageInterface $image) {
    // Preserve current background image dimensions.
    if ($image->getWidth() === 1 && $image->getHeight() === 1) {
      $image_width = 0;
      $image_height = 0;
    }
    else {
      $image_width = $image->getWidth();
      $image_height = $image->getHeight();
    }

    $this->info['image_width'] = $image_width;
    $this->info['image_height'] = $image_height;

    // Get the text wrapper Image object.
    if (!$wrapper = $this->getTextWrapper()) {
      return FALSE;
    }

    // Determine if background image needs resizing.
    if ($this->configuration['layout']['overflow_action'] == 'extend') {
      // The size of the frame sides for color filling.
      $this->info['frame_top'] = 0;
      $this->info['frame_right'] = 0;
      $this->info['frame_bottom'] = 0;
      $this->info['frame_left'] = 0;

      // Check wrapper image overflowing the original image.
      if ($this->canvasResizeNeeded($wrapper)) {
        // Apply textimage_define_canvas, transparent background.
        if (!$image->apply('textimage_define_canvas', [
                'exact' => [
                  'width' => $this->info['image_width'],
                  'height' => $this->info['image_height'],
                  'xpos' => $this->info['image_xpos'],
                  'ypos' => $this->info['image_ypos'],
                ],
              ]
            )) {
          return FALSE;
        }
        // Color fill the frame with carried on background color.
        if ($main_bg_color = $this->textimageFactory->getState('background_color')) {
          // Top rectangle.
          $rectangle = new Rectangle();
          if ($this->info['frame_top']) {
            $rectangle->setFromCorners([
              'c_a' => [0, $this->info['frame_top'] - 1],
              'c_b' => [$this->info['image_width'] - 1, $this->info['frame_top'] - 1],
              'c_c' => [$this->info['image_width'] - 1, 0],
              'c_d' => [0, 0],
            ]);
            if (!$image->apply('textimage_draw_rectangle', ['rectangle' => $rectangle, 'fill_color' => $main_bg_color])) {
              return FALSE;
            };
          }
          // Bottom rectangle.
          if ($this->info['frame_bottom']) {
            $rectangle->setFromCorners([
              'c_a' => [0, $this->info['image_height'] - 1],
              'c_b' => [$this->info['image_width'] - 1, $this->info['image_height'] - 1],
              'c_c' => [$this->info['image_width'] - 1, $image_height + $this->info['frame_top']],
              'c_d' => [0, $image_height + $this->info['frame_top']],
            ]);
            if (!$image->apply('textimage_draw_rectangle', ['rectangle' => $rectangle, 'fill_color' => $main_bg_color])) {
              return FALSE;
            };
          }
          // Left rectangle.
          if ($this->info['frame_left']) {
            $rectangle->setFromCorners([
              'c_a' => [0, $this->info['frame_top'] + $image_height - 1],
              'c_b' => [$this->info['frame_left'] - 1, $this->info['frame_top'] + $image_height - 1],
              'c_c' => [$this->info['frame_left'] - 1, $this->info['frame_top']],
              'c_d' => [0, $this->info['frame_top']],
            ]);
            if (!$image->apply('textimage_draw_rectangle', ['rectangle' => $rectangle, 'fill_color' => $main_bg_color])) {
              return FALSE;
            };
          }
          // Right rectangle.
          if ($this->info['frame_right']) {
            $rectangle->setFromCorners([
              'c_a' => [$this->info['frame_left'] + $image_width, $this->info['frame_top'] + $image_height - 1],
              'c_b' => [$this->info['image_width'] - 1, $this->info['frame_top'] + $image_height - 1],
              'c_c' => [$this->info['image_width'] - 1, $this->info['frame_top']],
              'c_d' => [$this->info['frame_left'] + $image_width, $this->info['frame_top']],
            ]);
            if (!$image->apply('textimage_draw_rectangle', ['rectangle' => $rectangle, 'fill_color' => $main_bg_color])) {
              return FALSE;
            };
          }
        }
      }
    }
    else {
      // Nothing to do, just place the wrapper at offset required.
      $x_offset = ceil(image_filter_keyword($this->configuration['layout']['x_pos'], $image_width, $wrapper->getWidth()));
      $y_offset = ceil(image_filter_keyword($this->configuration['layout']['y_pos'], $image_height, $wrapper->getHeight()));
      $this->info['wrapper_xpos'] = $x_offset + $this->configuration['layout']['x_offset'];
      $this->info['wrapper_ypos'] = $y_offset + $this->configuration['layout']['y_offset'];
    }

    // Finally, lay the wrapper over the source image.
    if (!$image->apply('textimage_overlay', ['layer' => $wrapper, 'x' => $this->info['wrapper_xpos'], 'y' => $this->info['wrapper_ypos']])) {
      return FALSE;
    }

    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function transformDimensions(array &$dimensions, $uri) {
    // Dimensions are potentially affected only if the effect is set to
    // autoextend the background image in case of wrapper overflow. Also,
    // current dimensions must be known.
    if ($dimensions['width'] && $dimensions['height'] && $this->configuration['layout']['overflow_action'] == 'extend') {

      $this->info['image_width'] = $dimensions['width'];
      $this->info['image_height'] = $dimensions['height'];

      // Get the text wrapper Image object.
      if (!$wrapper = $this->getTextWrapper()) {
        return;
      }

      // Checks if resizing needed.
      if ($this->canvasResizeNeeded($wrapper)) {
        $dimensions['width'] = $this->info['image_width'];
        $dimensions['height'] = $this->info['image_height'];
      }
    }
  }

  /**
   * Get the image containing the text.
   *
   * This is separated from ::applyEffect() so that it can also be used
   * by the ::transformDimensions() method.
   */
  protected function getTextWrapper() {
    // If the effect is executed outside of the context of Textimage
    // (e.g. by the core Image module), then the text_string has not been
    // pre-processed to translate tokens or apply text conversion.
    if (!($this->textimageFactory->getState('building_module') == 'textimage')) {
      $this->configuration['text_string'] = $this->textimageFactory->processTextString($this->configuration['text_string'], $this->configuration['text']['case_format'], [], new BubbleableMetadata()); // @todo check implications
    }

    // Create the wrapper image object from scratch.
    $wrapper = $this->imageFactory->get();

    // Return the wrapper built by the toolkit operation.
    $ret = $wrapper->apply('textimage_text_to_wrapper', [
      'font' => $this->configuration['font'],
      'layout' => $this->configuration['layout'],
      'text' => $this->configuration['text'],
      'text_string' => $this->configuration['text_string'],
      'debug_visuals' => isset($this->configuration['debug_visuals']) ? $this->configuration['debug_visuals'] : FALSE,
      'canvas_width' => $this->info['image_width'],
      'canvas_height' => $this->info['image_height'],
    ]);

    return $ret ? $wrapper : NULL;
  }

  /**
   * Recalculate background image size.
   *
   * When wrapper overflows the original image, and autoextent is set on.
   */
  protected function canvasResizeNeeded(ImageInterface $wrapper) {

    $resized = FALSE;

    // Background image dimensions.
    $image_width = $this->info['image_width'];
    $image_height = $this->info['image_height'];

    // Wrapper image dimensions.
    $wrapper_width = $wrapper->getWidth();
    $wrapper_height = $wrapper->getHeight();

    // Determine wrapper offset, based on placement option.
    // This is just taking into account the image and wrapper dimensions;
    // additional offset explicitly specified is considered later.
    $x_offset = ceil(image_filter_keyword($this->configuration['layout']['x_pos'], $image_width, $wrapper_width));
    $y_offset = ceil(image_filter_keyword($this->configuration['layout']['y_pos'], $image_height, $wrapper_height));

    // The position of the wrapper, once offset as per explicit
    // input. Width and height are not relevant for the algorithm,
    // but would be determined as follows:
    //  'width' => ($wrapper_width < $image_width) ? $wrapper_width + abs($this->configuration['layout']['x_offset']) : $wrapper_width;
    //  'height' = ($wrapper_height < $image_height) ? $wrapper_height + abs($this->configuration['layout']['y_offset']) : $wrapper_height;
    $this->info['wrapper_xpos'] = $x_offset + $this->configuration['layout']['x_offset'];
    $this->info['wrapper_ypos'] = $y_offset + $this->configuration['layout']['y_offset'];

    // If offset wrapper overflows to the left, background image
    // will be shifted to the right.
    if ($this->info['wrapper_xpos'] < 0) {
      $this->info['image_width'] = $image_width - $this->info['wrapper_xpos'];
      $this->info['image_xpos'] = -$this->info['wrapper_xpos'];
      $this->info['wrapper_xpos'] = 0;
      $this->info['frame_left'] = $this->info['image_width'] - $image_width;
      $resized = TRUE;
    }

    // If offset wrapper overflows to the top, background image
    // will be shifted to the bottom.
    if ($this->info['wrapper_ypos'] < 0) {
      $this->info['image_height'] = $image_height - $this->info['wrapper_ypos'];
      $this->info['image_ypos'] = -$this->info['wrapper_ypos'];
      $this->info['wrapper_ypos'] = 0;
      $this->info['frame_top'] = $this->info['image_height'] - $image_height;
      $resized = TRUE;
    }

    // If offset wrapper overflows to the right, background image
    // will be extended to the right.
    if (($this->info['wrapper_xpos'] + $wrapper_width) > $this->info['image_width']) {
      $tmp = $this->info['image_width'];
      $this->info['image_width'] = $this->info['wrapper_xpos'] + $wrapper_width;
      $this->info['frame_right'] = $this->info['image_width'] - $tmp;
      $resized = TRUE;
    }

    // If offset wrapper overflows to the bottom, background image
    // will be extended to the bottom.
    if (($this->info['wrapper_ypos'] + $wrapper_height) > $this->info['image_height']) {
      $tmp = $this->info['image_height'];
      $this->info['image_height'] = $this->info['wrapper_ypos'] + $wrapper_height;
      $this->info['frame_bottom'] = $this->info['image_height'] - $tmp;
      $resized = TRUE;
    }

    return $resized;
  }

  /**
   * Get the stroke mode for the font.
   *
   * @return string|null
   *   The stroke mode.
   */
  protected function strokeMode() {
    if ($this->configuration['font']['stroke_mode'] == 'outline' && ($this->configuration['font']['outline_top'] || $this->configuration['font']['outline_right'] || $this->configuration['font']['outline_bottom'] || $this->configuration['font']['outline_left'])) {
      return $this->t('Outline');
    }
    else if ($this->configuration['font']['stroke_mode'] == 'shadow') {
      return $this->t('Shadow');
    }
    else {
      return NULL;
    }
  }

}
